@@ -14,6 +14,8 @@ from six.moves import http_client
 from six.moves import map
 from six.moves import range
 
+from m3_gar_client.backends.m3_rest_gar.proxy.server import server
+from m3_gar_client.constants import GAR_HIERARCHY_MUN
 from m3_gar_client.constants import GAR_LEVELS_PLACE
 from m3_gar_client.constants import GAR_LEVELS_STREET
 from m3_gar_client.data import AddressObject
@@ -21,8 +23,6 @@ from m3_gar_client.data import House
 from m3_gar_client.data import ObjectMapper
 from m3_gar_client.utils import cached_property
 
-from .server import server
-
 
 # -----------------------------------------------------------------------------
 # Обёртки над данными, поступающими с сервера m3-rest-gar.
@@ -58,6 +58,11 @@ class AddressObjectMapper(ObjectMapper):
         'date_of_creation': 'startdate',
         'date_of_expiration': 'enddate',
         'full_name': lambda data: '{} {}'.format(data['name'], data['typename']),
+        'region_code': 'region_code',
+        'adm_parent_obj_id': 'hierarchy__adm__parentobjid__objectid',
+        'adm_parent_guid': 'hierarchy__adm__parentobjid__objectguid',
+        'mun_parent_obj_id': 'hierarchy__mun__parentobjid__objectid',
+        'mun_parent_guid': 'hierarchy__mun__parentobjid__objectguid',
     }
 
     assert set(fields_map) == set(AddressObject.fields)
@@ -87,6 +92,11 @@ class HouseMapper(ObjectMapper):
         'date_of_update': 'updatedate',
         'date_of_creation': 'startdate',
         'date_of_end': 'enddate',
+        'region_code': 'region_code',
+        'adm_parent_obj_id': 'hierarchy__adm__parentobjid__objectid',
+        'adm_parent_guid': 'hierarchy__adm__parentobjid__objectguid',
+        'mun_parent_obj_id': 'hierarchy__mun__parentobjid__objectid',
+        'mun_parent_guid': 'hierarchy__mun__parentobjid__objectguid',
     }
 
     assert set(fields_map) == set(House.fields)
@@ -99,8 +109,7 @@ class UIAddressObjectMapper(ObjectMapper):
     """
 
     fields_map = {
-        # TODO: заменить 'objectid' на 'objectguid'
-        'objectId': 'objectid',
+        'objectId': 'objectguid',
         'level': 'level',
         'shortName': 'typename',
         'formalName': 'name',
@@ -116,8 +125,7 @@ class UIHouseMapper(ObjectMapper):
     """
 
     fields_map = {
-        # TODO: заменить 'objectid' на 'objectguid'
-        'objectId': 'objectid',
+        'objectId': 'objectguid',
         'houseNumber': 'housenum',
         'buildingNumber': 'addnum1',
         'structureNumber': 'addnum2',
@@ -141,6 +149,7 @@ class LoaderBase(with_metaclass(ABCMeta, object)):
         """
         self.filter_string = filter_string
         self.timeout = kwargs.get('timeout')
+        self.exact = kwargs.get('exact', False)
 
     @property
     @abstractmethod
@@ -150,13 +159,13 @@ class LoaderBase(with_metaclass(ABCMeta, object)):
         :rtype: str
         """
 
-    @property
-    @abstractmethod
+    @cached_property
     def _fields(self):
         """Имена полей, подлежащих загрузке.
 
         :rtype: tuple
         """
+        return list(self._mapper_class.fields_map.keys())
 
     @property
     @abstractmethod
@@ -166,6 +175,14 @@ class LoaderBase(with_metaclass(ABCMeta, object)):
         ::rtype: m3_gar_client.data.ObjectMapper
         """
 
+    @property
+    def _filter_param(self):
+        """Query-параметр для фильтрации.
+
+        :rtype: str
+        """
+        return 'name'
+
     def _load_page(self, params, page):
         params = params.copy()
         params['page'] = page
@@ -207,13 +224,18 @@ class LoaderBase(with_metaclass(ABCMeta, object)):
         :rtype: bool
         """
 
-    def _get_params(self):  # pylint: disable=no-self-use
+    def _get_params(self):
         """Возвращает параметры запроса к серверу ГАР.
 
         :rtype: dict
         """
+        filter_param = self._filter_param
+
+        if self.exact:
+            filter_param += '__exact'
+
         result = {
-            'name': ','.join(word.replace('.', '') for word in self.filter_string.split()),
+            filter_param: self.filter_string,
         }
 
         return result
@@ -323,16 +345,12 @@ class AddressObjectLoaderBase(ParentFilterMixin, LoaderBase):
         * улицы.
     """
 
-    _default_hierarchy = 'mun'
+    _default_hierarchy = GAR_HIERARCHY_MUN
 
     @property
     def _path(self):
         return '/addrobj/'
 
-    @cached_property
-    def _fields(self):
-        return list(self._mapper_class.fields_map.keys())
-
     @property
     @abstractmethod
     def _levels(self):
@@ -351,6 +369,9 @@ class AddressObjectLoaderBase(ParentFilterMixin, LoaderBase):
         if self._levels:
             result['level'] = ','.join(map(str, self._levels))
 
+        if self._typenames:
+            result['typename'] = ','.join(map(str, self._typenames))
+
         return result
 
     def _filter(self, object_data):
@@ -372,7 +393,7 @@ class AddressObjectLoader(AddressObjectLoaderBase):
 
     _mapper_class = AddressObjectMapper
 
-    def __init__(self, filter_string, levels=None, **kwargs):
+    def __init__(self, filter_string, levels=None, typenames=None, **kwargs):
         """Инициализация экземпляра класса.
 
         :param unicode filter_string: Строка для фильтрации объектов.
@@ -381,6 +402,7 @@ class AddressObjectLoader(AddressObjectLoaderBase):
         super(AddressObjectLoader, self).__init__(filter_string, **kwargs)
 
         self._levels = levels
+        self._typenames = typenames
 
 
 class PlaceLoader(AddressObjectLoaderBase):
@@ -408,22 +430,24 @@ class HouseLoader(ParentFilterMixin, LoaderBase):
     Загрузчик сведений о зданиях.
     """
 
-    _default_hierarchy = 'mun'
+    _default_hierarchy = GAR_HIERARCHY_MUN
 
-    _mapper_class = UIHouseMapper
-
-    _fields = (
-        'objectId',
-        'houseNumber',
-        'buildingNumber',
-        'structureNumber',
-        'postalCode',
-    )
+    _mapper_class = HouseMapper
 
     @property
     def _path(self):
         return '/houses/'
 
+    @property
+    def _filter_param(self):
+        return 'housenum'
+
+    def _filter(self, object_data):
+        return True
+
+    def _sort_key(self, object_data):
+        return object_data['house_number'] or ''
+
     @staticmethod
     def _split_number(number):
         """Разделяет номер на целочисленную и буквенную части.
@@ -437,6 +461,10 @@ class HouseLoader(ParentFilterMixin, LoaderBase):
 
         return int(int_part) if int_part else -1, str_part
 
+
+class UIHouseLoader(HouseLoader):
+    _mapper_class = UIHouseMapper
+
     def _sort_key(self, object_data):
         return tuple(chain(*(
             self._split_number(object_data[number_type + 'Number'])
@@ -476,7 +504,7 @@ class HouseLoader(ParentFilterMixin, LoaderBase):
         return result
 
     def _process_object_data(self, drf_object_data):
-        house_data = super(HouseLoader, self)._process_object_data(drf_object_data)
+        house_data = super(UIHouseLoader, self)._process_object_data(drf_object_data)
 
         house_data['houseNumber'] = house_data['houseNumber'] or ''
         house_data['buildingNumber'] = house_data['buildingNumber'] or ''
@@ -484,7 +512,6 @@ class HouseLoader(ParentFilterMixin, LoaderBase):
 
         return house_data
 
-
 # -----------------------------------------------------------------------------
 # Функции для создания объектов m3-gar-client на основе данных m3-rest-gar.
 
@@ -493,7 +520,7 @@ def get_address_object(obj_id, timeout=None):
 
     Если адресный объект не найден, возвращает ``None``.
 
-    :param obj_id: ID адресного объекта ГАР.
+    :param obj_id: ObjectID/GUID адресного объекта ГАР.
     :param float timeout: timeout запроса к серверу ГАР в секундах.
 
     :rtype: m3_gar_client.data.AddressObject or NoneType
@@ -502,10 +529,6 @@ def get_address_object(obj_id, timeout=None):
     """
     assert obj_id is not None
 
-    # TODO: Сервер ГАР пока не поддерживает поиск адресного объекта по objectguid.
-    # В идеале нужно смотреть на вид obj_id и вызывать разные методы поиска в зависимости от него
-    # Пока предполагаем, что obj_id - число (int).
-
     response = server.get('/addrobj/{}/'.format(obj_id), timeout=timeout)
 
     if response.status_code == http_client.OK:
@@ -518,7 +541,13 @@ def get_address_object(obj_id, timeout=None):
     return result
 
 
-def find_address_objects(filter_string, levels=None, parent_id=None, timeout=None):
+def find_address_objects(
+    filter_string,
+    levels=None,
+    typenames=None,
+    parent_id=None,
+    timeout=None,
+):
     """Возвращает адресные объекты, соответствующие параметрам поиска.
 
     :param unicode filter_string: Строка поиска.
@@ -528,7 +557,13 @@ def find_address_objects(filter_string, levels=None, parent_id=None, timeout=Non
 
     :rtype: generator
     """
-    return AddressObjectLoader(filter_string, levels=levels, parent_id=parent_id, timeout=timeout).load_results()
+    return AddressObjectLoader(
+        filter_string,
+        levels=levels,
+        typenames=typenames,
+        parent_id=parent_id,
+        timeout=timeout,
+    ).load_results()
 
 
 def get_house(house_id, timeout=None):
@@ -553,38 +588,24 @@ def get_house(house_id, timeout=None):
     return result
 
 
-def find_house(obj_id, house_number='', building_number='', structure_number='', timeout=None):
+def find_house(house_number='', parent_id=None, timeout=None):
     """Возвращает информацию о здании по его номеру.
 
-    :param obj_id: ID адресного объекта.
     :param unicode house_number: Номер дома.
-    :param unicode building_number: Номер корпуса.
-    :param unicode structure_number: Номер строения.
+    :param parent_id: ID родительского объекта.
     :param float timeout: Timeout запросов к серверу ГАР в секундах.
 
     :rtype: m3_gar_client.data.House or NoneType
     """
-    assert house_number or building_number or structure_number, (
-        house_number, building_number, structure_number
-    )
-
-    houses = tuple(
-        house_info['objectId']
-        for house_info in HouseLoader(
-            obj_id,
-            house_number or building_number or structure_number,
-            timeout=timeout,
-        ).load()
-        if (
-            house_info['objectId'] and
-            house_info['houseNumber'] == (house_number or '') and
-            house_info['buildingNumber'] == (building_number or '') and
-            house_info['structureNumber'] == (structure_number or '')
-        )
-    )
+    houses = HouseLoader(
+        house_number,
+        parent_id=parent_id,
+        exact=True,
+        timeout=timeout,
+    ).load()
 
     if len(houses) == 1:
-        result = get_house(houses[0], timeout)
+        result = House(**houses[0])
     else:
         result = None
 
