@@ -1,6 +1,8 @@
 # coding: utf-8
 from __future__ import absolute_import
 from __future__ import unicode_literals
+
+import json
 import warnings
 
 from m3_ext.ui.base import BaseExtComponent
@@ -11,19 +13,18 @@ from m3_ext.ui.fields.simple import ExtStringField
 from m3_ext.ui.fields.simple import ExtTextArea
 from m3_ext.ui.misc.store import ExtJsonStore
 
+from m3_gar_client.constants import UI_LEVEL_FLAT
+from m3_gar_client.constants import UI_LEVEL_HOUSE
+from m3_gar_client.constants import UI_LEVEL_PLACE
+from m3_gar_client.constants import UI_LEVEL_STREET
+from m3_gar_client.constants import UI_LEVELS
 from m3_gar_client.data import ObjectDictAdapter
+from m3_gar_client.data import ObjectMapper
+from m3_gar_client.utils import cached_property
 from m3_gar_client.utils import get_address_object
 from m3_gar_client.utils import get_house
 import m3_gar_client
 
-from .constants import UI_LEVEL_FLAT
-from .constants import UI_LEVEL_HOUSE
-from .constants import UI_LEVEL_PLACE
-from .constants import UI_LEVEL_STREET
-from .constants import UI_LEVELS
-from .data import ObjectMapper
-from .utils import cached_property
-
 
 class UIAddressObjectMapper(ObjectMapper):
     """Обертка над адресным объектом для передачи данных в UI.
@@ -33,8 +34,7 @@ class UIAddressObjectMapper(ObjectMapper):
     """
 
     fields_map = {
-        # TODO: заменить 'obj_id' на 'guid'
-        'objectId': 'obj_id',
+        'objectId': 'guid',
         'level': 'level',
         'shortName': 'short_name',
         'formalName': 'formal_name',
@@ -51,8 +51,7 @@ class UIHouseMapper(ObjectMapper):
     """
 
     fields_map = {
-        # TODO: заменить 'obj_id' на 'guid'
-        'objectId': 'obj_id',
+        'objectId': 'guid',
         'houseNumber': 'house_number',
         'buildingNumber': 'building_number',
         'structureNumber': 'structure_number',
@@ -706,6 +705,36 @@ class AdvancedRowsAddressView(RowsAddressView):
 
     structure_field_name = property(get_structure_field_name, set_structure_field_name)
 
+    @classmethod
+    def addr_object(cls, code):
+        """
+        Информация об адресном объекте для ручного биндинга значений в поля
+        """
+        obj = get_address_object(code)
+
+        data = {
+            'address': '',
+            'ao_guid': '',
+            'ao_level': '',
+            'formal_name': '',
+            'name': '',
+            'place_address': '',
+            'postal_code': '',
+            'shortname': '',
+        }
+
+        if obj:
+            data.update({
+                'ao_guid': str(obj.guid),
+                'ao_level': obj.level,
+                'formal_name': obj.formal_name,
+                'name': obj.formal_name,
+                'postal_code': obj.postal_code,
+                'shortname': obj.short_name,
+            })
+
+        return json.dumps(data)
+
     def get_zipcode(self):
         """
         Получаем значение почтового индекса. Если он явно не указан, то берем
