@@ -2,8 +2,10 @@
 from __future__ import absolute_import
 from __future__ import unicode_literals
 
-from .data import AddressObject
-from .data import House
+from m3_gar_client.constants import GAR_HIERARCHIES
+from m3_gar_client.constants import GAR_HIERARCHY_MUN
+from m3_gar_client.data import AddressObject
+from m3_gar_client.data import House
 
 
 class cached_property(property):
@@ -75,7 +77,7 @@ def correct_keyboard_layout(text):
     return result
 
 
-def find_address_objects(filter_string, levels=None, parent_id=None, timeout=None):
+def find_address_objects(filter_string, levels=None, typenames=None, parent_id=None, timeout=None):
     """Возвращает адресные объекты, соответствующие параметрам поиска.
 
     :param unicode filter_string: Строка поиска.
@@ -87,7 +89,7 @@ def find_address_objects(filter_string, levels=None, parent_id=None, timeout=Non
     """
     from m3_gar_client import config
 
-    return config.backend.find_address_objects(filter_string, levels, parent_id, timeout)
+    return config.backend.find_address_objects(filter_string, levels, typenames, parent_id, timeout)
 
 
 def get_address_object(obj_id, timeout=None):
@@ -103,20 +105,18 @@ def get_address_object(obj_id, timeout=None):
     return config.backend.get_address_object(obj_id, timeout)
 
 
-def find_house(obj_id, house_number='', building_number='', structure_number='', timeout=None):
+def find_house(house_number='', parent_id=None, timeout=None):
     """Возвращает информацию о здании по его номеру.
 
-    :param obj_id: ID адресного объекта.
     :param unicode house_number: Номер дома.
-    :param unicode building_number: Номер корпуса.
-    :param unicode structure_number: Номер строения.
+    :param parent_id: ID родительского объекта.
     :param float timeout: Timeout запросов к серверу ГАР в секундах.
 
     :rtype: m3_gar_client.data.House or NoneType
     """
     from m3_gar_client import config
 
-    return config.backend.find_house(obj_id, house_number, building_number, structure_number, timeout)
+    return config.backend.find_house(house_number, parent_id, timeout)
 
 
 def get_house(house_id, parent_id=None, timeout=None):
@@ -193,7 +193,7 @@ def get_house_name(house):
     return ', '.join(names)
 
 
-def get_full_name(obj, timeout=None):
+def get_full_name(obj, hierarchy=GAR_HIERARCHY_MUN, with_postal_code=True, timeout=None):
     """Возвращает полное наименование адресного объекта или здания.
 
     Примеры:
@@ -213,27 +213,63 @@ def get_full_name(obj, timeout=None):
     if not isinstance(obj, (House, AddressObject)):
         raise TypeError(obj)
 
+    for addrobj in get_full_details(obj, hierarchy=hierarchy):
+        if postal_code is None and addrobj.postal_code:
+            postal_code = addrobj.postal_code
+
+        if isinstance(addrobj, House):
+            names.append(get_house_name(addrobj))
+
+        elif isinstance(addrobj, AddressObject):
+            names.append(get_address_object_name(addrobj))
+
+    if with_postal_code and postal_code is not None:
+        names.insert(0, postal_code)
+
+    full_name = ', '.join(reversed(names))
+
+    return full_name
+
+
+def get_full_details(obj, hierarchy=GAR_HIERARCHY_MUN, timeout=None):
+    """
+    Возвращает полный список объектов, снизу вверх из иерархии.
+    """
+    if not isinstance(obj, (House, AddressObject)):
+        raise TypeError(obj)
+
+    if hierarchy not in GAR_HIERARCHIES:
+        raise ValueError(hierarchy)
+
+    parent_attr = f'{hierarchy}_parent_obj_id'
+
+    objects = []
+
     if isinstance(obj, House):
-        if obj.postal_code:
-            postal_code = obj.postal_code
+        objects.append(obj)
 
-        names.append(get_house_name(obj))
-
-        obj = get_address_object(obj.parent_id, timeout)
+        parent_id = getattr(obj, parent_attr)
+        if parent_id:
+            obj = get_address_object(parent_id, timeout)
 
     if isinstance(obj, AddressObject):
         while obj:
-            if postal_code is None and obj.postal_code:
-                postal_code = obj.postal_code
+            objects.append(obj)
 
-            names.insert(0, get_address_object_name(obj))
-
-            if obj.parent_id:
-                obj = get_address_object(obj.parent_id, timeout)
+            parent_id = getattr(obj, parent_attr)
+            if parent_id:
+                obj = get_address_object(parent_id, timeout)
             else:
                 break
 
-    if postal_code is not None:
-        names.insert(0, postal_code)
+    return objects
 
-    return ', '.join(names)
+
+def get_details(obj, timeout=None):
+    """
+    Возвращает список наименований и сокращений узла, снизу вверх из иерархии.
+    Не включая верхний уровень "Субъект РФ"
+    """
+    result = get_full_details(obj, timeout=timeout)
+
+    return result[1:] if len(result) > 1 else result
